@@ -1,22 +1,28 @@
 from typing import Dict, Tuple
-from polaris.utils import getLogger
+from decimal import Decimal, DecimalException
 
 from django.utils.translation import gettext as _
+from django.core.validators import URLValidator
+from django.core.exceptions import ValidationError
 from rest_framework.request import Request
 from rest_framework.response import Response
-from rest_framework.decorators import api_view, renderer_classes
-from rest_framework.renderers import JSONRenderer
+from rest_framework.decorators import api_view, renderer_classes, parser_classes
+from rest_framework.parsers import JSONParser, MultiPartParser, FormParser
+from rest_framework.renderers import JSONRenderer, BrowsableAPIRenderer
 from stellar_sdk.exceptions import MemoInvalidException
 
+from polaris import settings
 from polaris.utils import (
+    getLogger,
     render_error_response,
     create_transaction_id,
     extract_sep9_fields,
-    memo_str,
+    make_memo,
     memo_hex_to_base64,
 )
 from polaris.sep6.utils import validate_403_response
 from polaris.sep10.utils import validate_sep10_token
+from polaris.shared.endpoints import SEP6_MORE_INFO_PATH
 from polaris.locale.utils import validate_language, activate_lang_for_request
 from polaris.models import Asset, Transaction
 from polaris.integrations import (
@@ -25,13 +31,13 @@ from polaris.integrations import (
     calculate_fee,
 )
 
-
 logger = getLogger(__name__)
 
 
 @api_view(["GET"])
-@renderer_classes([JSONRenderer])
-@validate_sep10_token("sep6")
+@parser_classes([MultiPartParser, FormParser, JSONParser])
+@renderer_classes([JSONRenderer, BrowsableAPIRenderer])
+@validate_sep10_token()
 def withdraw(account: str, request: Request) -> Response:
     args = parse_request_args(request)
     if "error" in args:
@@ -41,7 +47,7 @@ def withdraw(account: str, request: Request) -> Response:
     transaction_id = create_transaction_id()
     transaction_id_hex = transaction_id.hex
     padded_hex_memo = "0" * (64 - len(transaction_id_hex)) + transaction_id_hex
-    memo = memo_hex_to_base64(padded_hex_memo)
+    transaction_memo = memo_hex_to_base64(padded_hex_memo)
     transaction = Transaction(
         id=transaction_id,
         stellar_account=account,
@@ -49,9 +55,13 @@ def withdraw(account: str, request: Request) -> Response:
         kind=Transaction.KIND.withdrawal,
         status=Transaction.STATUS.pending_user_transfer_start,
         receiving_anchor_account=args["asset"].distribution_account,
-        memo=memo,
+        memo=transaction_memo,
         memo_type=Transaction.MEMO_TYPES.hash,
         protocol=Transaction.PROTOCOL.sep6,
+        more_info_url=request.build_absolute_uri(
+            f"{SEP6_MORE_INFO_PATH}?id={transaction_id}"
+        ),
+        on_change_callback=args["on_change_callback"],
     )
 
     # All request arguments are validated in parse_request_args()
@@ -72,9 +82,9 @@ def withdraw(account: str, request: Request) -> Response:
         )
 
     if status_code == 200:
-        response["memo"] = memo
-        response["memo_type"] = Transaction.MEMO_TYPES.hash
-        logger.info(f"Created withdraw transaction {transaction_id}")
+        response["memo"] = transaction.memo
+        response["memo_type"] = transaction.memo_type
+        logger.info(f"Created withdraw transaction {transaction.id}")
         transaction.save()
     elif Transaction.objects.filter(id=transaction.id).exists():
         logger.error("Do not save transaction objects for invalid SEP-6 requests")
@@ -104,7 +114,7 @@ def parse_request_args(request: Request) -> Dict:
         return {"error": render_error_response(_("invalid 'memo_type'"))}
 
     try:
-        memo = memo_str(request.GET.get("memo"), memo_type)
+        make_memo(request.GET.get("memo"), memo_type)
     except (ValueError, MemoInvalidException):
         return {"error": render_error_response(_("invalid 'memo' for 'memo_type'"))}
 
@@ -113,14 +123,45 @@ def parse_request_args(request: Request) -> Dict:
     if not request.GET.get("dest"):
         return {"error": render_error_response(_("'dest' is required"))}
 
+    on_change_callback = request.GET.get("on_change_callback")
+    if on_change_callback and on_change_callback.lower() != "postmessage":
+        schemes = ["https"] if not settings.LOCAL_MODE else ["https", "http"]
+        try:
+            URLValidator(schemes=schemes)(on_change_callback)
+        except ValidationError:
+            return {"error": render_error_response(_("invalid callback URL provided"))}
+        if any(
+            domain in on_change_callback
+            for domain in settings.CALLBACK_REQUEST_DOMAIN_DENYLIST
+        ):
+            on_change_callback = None
+
+    amount = request.GET.get("amount")
+    if amount:
+        try:
+            amount = round(Decimal(amount), asset.significant_decimals)
+        except DecimalException:
+            return {"error": render_error_response(_("invalid 'amount'"))}
+        min_amount = round(asset.withdrawal_min_amount, asset.significant_decimals)
+        max_amount = round(asset.withdrawal_max_amount, asset.significant_decimals)
+        if not (min_amount <= amount <= max_amount):
+            return {
+                "error": render_error_response(
+                    _("'amount' must be within [%s, %s]") % (min_amount, min_amount)
+                )
+            }
+
     args = {
         "asset": asset,
         "memo_type": memo_type,
-        "memo": memo,
+        "memo": request.GET.get("memo"),
         "lang": request.GET.get("lang"),
         "type": request.GET.get("type"),
         "dest": request.GET.get("dest"),
         "dest_extra": request.GET.get("dest_extra"),
+        "on_change_callback": on_change_callback,
+        "amount": amount,
+        "country_code": request.GET.get("country_code"),
         **extract_sep9_fields(request.GET),
     }
 
@@ -135,15 +176,15 @@ def parse_request_args(request: Request) -> Dict:
 def validate_response(
     args: Dict, integration_response: Dict, transaction: Transaction
 ) -> Tuple[Dict, int]:
-    account = args["account"]
-    asset = args["asset"]
     if "type" in integration_response:
-        return validate_403_response(account, integration_response, transaction), 403
+        return validate_403_response(integration_response, transaction), 403
 
+    asset = args["asset"]
     response = {
         "account_id": asset.distribution_account,
         "min_amount": round(asset.withdrawal_min_amount, asset.significant_decimals),
         "max_amount": round(asset.withdrawal_max_amount, asset.significant_decimals),
+        "id": transaction.id,
     }
 
     if calculate_fee == registered_fee_func:
